@@ -26,6 +26,44 @@
 ##
 ##  31 mai  2023 : si une colonne d'identifiant est présente, mais vide,
 ##                  elle est ignorée avec un avertissement
+##
+##  24 juin 2023 : le c ne sert plus pour détecter les catégories
+##                 questions numériques à réponses multiples possible [format QCM]
+##                 format des réponses numériques plus riche : tolérance absolue possible [±]
+##
+##  12 jan. 2024 : bilan des types de questions converties, en fin de conversion
+##                 traitement des notes pour ôter les espaces, si en texte…
+##
+##  14 fév. 2024 : correction d'un message dans le résumé final
+##
+##  12 avr. 2024 : correction d'un accent dans un avertissement
+##                 erreur en cas d'absence de bonne réponse dans une question
+##
+##  15 avr. 2024 : mémorisation de la data.frame en cours de conversion
+##                  ⇒ permet de la récupérer pour explorer l'erreur…
+##
+##  23 avr. 2024 : type QCI : qcm à notation « internat »
+##
+##  15 mai  2024 : corrigé la correction des notes « textes » : fonctionne
+##                    même si pas de colonne de notes fournie…
+##
+##  29 mai  2024 : corrigé l'affichage des réponses numériques
+##                 corrigé la détection des QCM internats quand aucun type de QCM indiqué
+##
+##   3 jui. 2024 : si le séparateur est à NA, on ne fait pas de conversion
+##
+##  17 jui. 2024 : on ôte les lignes NA pour faire le décompte des questions par type
+##
+##   3 fév. 2025 : on indique si la réponse trouvée est entière
+##                 df.moodle() peut ouvrir le fichier XML, si indiqué comme chaîne
+##                 si pas de dossier.image, considéré vecteur vide et non missing
+##
+##  18 juin 2025 : les textes à trous sont bien identifiés
+##                 suppression des doubles « Attention »
+##                 suppression d'un espace indû après ( dans le bilan si tout en catégorie…
+##                 décompte correct des questions à réponses multiples
+##
+##  19 juin 2025 : pour les questions numériques simples, tolérance prise en compte
 ## ——————————————————————————————————————————————————————————————————————
 
 ## ——————————————————————————————————————————————————————————————————————
@@ -91,6 +129,8 @@ df.moodle <- function( d,
                        sep.SMILES = c( '@\\{', '\\}@' ),
                        nom.fichier = deparse( substitute( d ) ) )
 {
+    assign( "df.convertie", list( "Brute" = d ), envir = SARP.Moodle.env )
+
     if ( nrow( d ) < 1 ) {
         erreur( 1000, "df.moodle",
                 "Aucune question, conversion annul\u00e9e." )
@@ -100,6 +140,12 @@ df.moodle <- function( d,
                 "Il faut au moins deux colonnes, \u00e9nonc\u00e9s et r\u00e9ponses." )
     }
 
+    ## On ouvre le fichier, au besoin
+    nv.fichier <- is.character( fichier.xml )
+    if ( nv.fichier ) {
+        fichier.xml <- debuter_xml.moodle( fichier.xml )
+    }
+    
     ## On contrôle la précision
     if ( missing( precision ) ) precision <- 3
     if ( is.null( precision ) ) precision <- 3
@@ -123,7 +169,8 @@ df.moodle <- function( d,
         erreur( 5, "df.moodle",
                 "inserer.images ne doit avoir qu'une seule valeur, TRUE ou FALSE" )
     }
-    
+
+    if ( missing( dossier.images ) ) dossier.images <- character()
     if ( length( dossier.images ) > 1 ) {
         erreur( 6, "df.moodle",
                 "Un seul dossier contenant les images",
@@ -233,18 +280,18 @@ df.moodle <- function( d,
     ##   (remarque : si absente, NA : is.integer vaut FALSE
     ##                                sauf si NA_integer_ mais alors noms.colonnes[ ] renvoie NA
     ##    ==> devrait fonctionner...)
-    if ( is.integer( colonne.texte   ) ) colonne.texte   <- noms.colonnes[ colonne.texte   ]
-    if ( is.integer( colonne.reponse ) ) colonne.reponse <- noms.colonnes[ colonne.reponse ]
-    if ( is.integer( colonne.note    ) ) colonne.note    <- noms.colonnes[ colonne.note    ]
+    if ( is.integer( colonne.texte    ) ) colonne.texte   <- noms.colonnes[ colonne.texte   ]
+    if ( is.integer( colonne.reponse  ) ) colonne.reponse <- noms.colonnes[ colonne.reponse ]
+    if ( is.integer( colonne.note     ) ) colonne.note    <- noms.colonnes[ colonne.note    ]
     if ( is.integer( colonne.note_question ) ) colonne.note_question <- noms.colonnes[ colonne.note_question ]
-    if ( is.integer( colonne.titre   ) ) colonne.titre   <- noms.colonnes[ colonne.titre   ]
-    if ( is.integer( colonne.code    ) ) colonne.code    <- noms.colonnes[ colonne.code    ]
-    if ( is.integer( colonne.type    ) ) colonne.type    <- noms.colonnes[ colonne.type    ]
-    if ( is.integer( colonne.retour  ) ) colonne.retour  <- noms.colonnes[ colonne.retour  ]
-    if ( is.integer( colonne.global  ) ) colonne.global  <- noms.colonnes[ colonne.global  ]
+    if ( is.integer( colonne.titre    ) ) colonne.titre   <- noms.colonnes[ colonne.titre   ]
+    if ( is.integer( colonne.code     ) ) colonne.code    <- noms.colonnes[ colonne.code    ]
+    if ( is.integer( colonne.type     ) ) colonne.type    <- noms.colonnes[ colonne.type    ]
+    if ( is.integer( colonne.retour   ) ) colonne.retour  <- noms.colonnes[ colonne.retour  ]
+    if ( is.integer( colonne.global   ) ) colonne.global  <- noms.colonnes[ colonne.global  ]
     if ( is.integer( colonne.penalite ) ) colonne.penalite <- noms.colonnes[ colonne.penalite ]
-    if ( is.integer( colonne.temps   ) ) colonne.temps   <- noms.colonnes[ colonne.temps ]
-    if ( is.integer( colonne.temps   ) ) colonne.temps   <- noms.colonnes[ colonne.temps ]
+    if ( is.integer( colonne.temps    ) ) colonne.temps   <- noms.colonnes[ colonne.temps ]
+    if ( is.integer( colonne.temps    ) ) colonne.temps   <- noms.colonnes[ colonne.temps ]
 
     cat( "  Colonne des \u00e9nonc\u00e9s        :", colonne.texte  , "\n" )
     cat( "  Colonne des r\u00e9ponses       :"     , colonne.reponse, "\n" )
@@ -274,7 +321,7 @@ df.moodle <- function( d,
                        n, " case", if ( n > 1 ) "s",
                        " avec seulement des espaces",
                        " dans les \u00e9nonc\u00e9s -",
-                       " consid\ue009r\u00e9e", if ( n > 1 ) "s",
+                       " consid\u00e9r\u00e9e", if ( n > 1 ) "s",
                        " vide", if ( n > 1 ) "s", "." )
         d[ idx, colonne.texte ] <- NA
     } 
@@ -391,7 +438,7 @@ df.moodle <- function( d,
 
         d$I.Categorie[ which( ( d[ , colonne.type ] %in% c( 'categorie',
                                                             'category',
-                                                            'c', 'cat' ) ) &
+                                                            'cat' ) ) &
                               ( d$I_Code.interne %in% id.simple ) ) ] <- TRUE
     }
     n.categories <- length( which( d$I.Categorie ) )
@@ -416,7 +463,7 @@ df.moodle <- function( d,
         }
         cat( "\n\n" )
         categorie.moodle( nom.categorie = categorie,
-                         fichier.xml = fichier.xml )
+                          fichier.xml = fichier.xml )
     } else {
         cat( "\n Cat\u00e9gorie", if ( n.categories > 1 ) "s",
              " donn\u00e9e", if ( n.categories > 1 ) "s",
@@ -444,7 +491,9 @@ df.moodle <- function( d,
     }
     
     ## On adapte les textes des questions : images...
-    if ( all( length( dossier.images ) > 0, length( sep.images ) == 2 ) ) { 
+    if ( all( length( dossier.images ) > 0,
+              length( sep.images ) == 2,
+              !is.na( sep.images ) ) ) { 
         cat( " Conversion des images...\n" )
         
         d[ , colonne.texte   ] <- traiter_images( d[ , colonne.texte   ], 
@@ -461,7 +510,8 @@ df.moodle <- function( d,
         }
     }
 
-    if ( all( length( sep.formules ) == 2 ) ) { 
+    if ( all( length( sep.formules ) == 2,
+              !is.na( sep.formules ) ) ) {
         cat( " Conversion des formules...\n" )
         
         d[ , colonne.texte   ] <- traiter_formules( d[ , colonne.texte   ], sep.formules ) # Énoncé
@@ -474,7 +524,8 @@ df.moodle <- function( d,
         }
     }
 
-    if ( all( length( sep.SMILES ) == 2 ) ) { 
+    if ( all( length( sep.SMILES ) == 2,
+              !is.na( sep.SMILES ) ) ) {
         cat( " Conversion des codes SMILES...\n" )
         
         d[ , colonne.texte   ] <- traiter_SMILES( d[ , colonne.texte   ], sep.SMILES ) # Énoncé
@@ -499,17 +550,30 @@ df.moodle <- function( d,
     if ( !is.na( colonne.global ) ) {
         d[ , colonne.global  ] <- preparer_texte( d[ , colonne.global  ] ) # Commentaire global si faux
     }
+    if ( !is.na( colonne.note ) ) {
+        if ( is.character( d[ , colonne.note ] ) ) {
+            d[ , colonne.note  ] <- preparer_texte( d[ , colonne.note  ],
+                                                   oter.insecables = TRUE ) # Note : peut contenir VRAI/FAUX et donc pb d'espace...
+        }
+    }
 
     ## On prépare les colonnes internes recevant le résultat
     d$Avertissements__ <- NA_character_
     d$Type.final__ <- NA_character_
 
+    lst <- get( "df.convertie", envir = SARP.Moodle.env)
+    lst$Preparee = d
+    assign( "df.convertie", lst, envir = SARP.Moodle.env )
+
     ## On travaille par question…
     for( q in codes ) {
         cat( " Conversion de la question ", q )
         idx.question <- which( d$I_Code.interne == q )
         dq <- d[ idx.question, ]
 
+        lst$Question = dq
+        assign( "df.convertie", lst, envir = SARP.Moodle.env )
+
         avertissements.avant <- get( "liste.avertissements", envir = SARP.Moodle.env )
         if ( nrow( dq ) == 1 ) {
             cat( " [simple]\n" )
@@ -566,6 +630,47 @@ df.moodle <- function( d,
             d$Avertissements__[ idx.question ] <- paste0( avertissements.apres[ ( length( avertissements.avant ) + 1 ):length( avertissements.apres ) ], collapse = ", " )
         }
     }
+
+    ## On fait un bilan de la conversion réalisée...
+    idx.OK <- which( !is.na( d$Numero.interne ) )
+    n.questions <- length( unique( d$Numero.interne[ idx.OK ] ) )
+    cat( sep = "", "\r\n",
+         "Nombre de questions trait\u00e9es : ", n.questions )
+    if ( n.categories == 0 ) {
+        cat( sep = "", " (sans indication de cat\u00e9gorie)" )
+    } else {
+        cat( sep = "", " (" )
+        if ( d$I.Categorie[ 1 ] == FALSE ) {
+            idx.C1 <- which( d$I.Categorie )[ 1 ]
+            num <- unique( d$Numero.interne[ 1:idx.C1 ] )
+            if ( any( is.na( num ) ) ) num <- num[ -which( is.na( num ) ) ]
+            n.vrac <- length( num )
+            n.autres <- n.questions - n.vrac
+            cat( sep = "", "dont ", n.vrac, " sans indication,",
+                 " ", if ( n.autres > 1 ) "les autres" else "la derni\u00e8re",
+                 " " )
+        } else {
+            n.autres <- n.questions
+        }
+        if ( n.categories > 1 ) {
+            cat( sep = "", "r\u00e9partie", if ( n.autres > 1 ) "s", " en" )
+        } else {
+            cat( sep = "", "plac\u00e9e", if ( n.autres > 1 ) "s", " dans" )
+        }
+        cat( sep = "", 
+             " ", n.categories, " cat\u00e9gorie", if ( n.categories > 1 ) "s",
+             ")" )
+    }
+    cat( sep = "", " dont" )
+
+    idx.l1 <- which( !duplicated( d$Numero.interne ) &
+                     !is.na     ( d$Numero.interne ) )
+    print( table( d$Type.final__[ idx.l1 ] ) )
+
+    ## On a fini : on ferme le fichier au besoin
+    if ( TRUE == nv.fichier ) {
+        finir_xml.moodle( fichier.xml )
+    }
     
     ## On renvoie la base des questions...
     d
@@ -702,7 +807,7 @@ convertir.question_simple <- function( question,
                                        note.question = note.question,
                                        penalite = penalite.question,
                                        fichier.xml = fichier.xml )
-                return( "Description" )
+                return( "\u00c0 trou" )
             }
         }
 
@@ -728,18 +833,20 @@ convertir.question_simple <- function( question,
     }
 
     if ( question$I.Categorie ) {
-        erreur( 1006, convertir.question_simple,
+        erreur( 1006, "convertir.question_simple",
                 "Question identifi\u00e9e \u00e0 tort comme cat\u00e9gorie !" )
     }
 
     ## Cas 2 : réponse numérique
     ##   (en évitant des avertissements superflus)
-    reponse.n <- suppressWarnings( as.double( reponse ) )
-    if ( is.na( reponse.n ) )
-        reponse.n <- suppressWarnings( as.double( gsub( ",", ".", fixed = TRUE, reponse ) ) )
-    if ( is.finite( reponse.n ) ) {
+    reponse.n <- trouver_reponse.numerique( reponse, "convertir.question_simple" )
+    if ( is.finite( reponse.n$Nombre ) ) {
         cat( "  Question simple, r\u00e9ponse num\u00e9rique :",
-             reponse.n, "\n" )
+             as.character( reponse.n$Nombre ),
+             if ( is.integer( reponse.n$Nombre ) ) "[enti\u00e8re]",
+             if ( is.finite( reponse.n$Tolerance ) ) paste0( "\u00b1 ", 
+                                                             reponse.n$Tolerance ),
+             "\n" )
 
         ## Quel nombre de décimales pour la réponse ?
         precision <- NA
@@ -750,39 +857,25 @@ convertir.question_simple <- function( question,
 
         ## 2) détection à partir du nombre de chiffres après la virgule
         if ( is.na( precision ) ) {
-            pos.decimale <- regexpr( ",|\\.", reponse )
-            if( pos.decimale > 0 ) {
-                decimales <- substr( reponse, pos.decimale + 1, nchar( reponse ) )
-                decimales <- gsub( "[[:space:]]", "", decimales )
-                if ( grepl( "e[-+0-9]+", decimales, ignore.case = TRUE ) ) {
-                    avertissement( 1001, "convertir.question_simple",
-                                   "Nombre en notation scientifique,",
-                                   " la d\u00e9tection automatique du nombre de d\u00e9cimales",
-                                   " risque d'\u00e9chouer.",
-                                   " Utilisez la colonne Pr\u00e9cision." )
-                    decimales <- gsub( "e[-+0-9]+", "", decimales, ignore.case = TRUE )
-                }
-                precision <- nchar( decimales )
-            } else {
-                reponse.n <- as.integer( reponse.n ) # Entier => pas de message
-            }
+            precision <- reponse.n$Precision
         }
 
         ## 3) valeur par défaut, si réponse non-entière
         if ( is.na( precision ) ) {
-            if ( ( reponse.n - as.integer( reponse.n ) ) == 0 ) {
+            if ( all( ( reponse.n$Nombre - as.integer( reponse.n$Nombre ) ) == 0 ) ) {
                 ## Réponse apparemment entière : pas d'indication sur le nombre de décimales
                 precision <- NA
-                reponse.n <- as.integer( reponse.n )
+                reponse.n$Nombre <- as.integer( reponse.n$Nombre )
             } else {
                 precision <- precision.defaut
             }
         }
 
         numerique.moodle( texte = question[ 1, colonne.texte ],
-                          bonne.reponse = reponse.n,
+                          bonne.reponse = reponse.n$Nombre,
                           titre = question[ 1, colonne.titre ],
                           n.decimales = precision,
+                          tolerance = reponse.n$Tolerance, tolerance.type = 2,
                           temps = temps,
                           fichier.xml = fichier.xml,
                           commentaire.global = commentaire.global,
@@ -980,9 +1073,6 @@ convertir_question <- function( question,
     ## Combien de réponses à la question
     n.reponses <- nrow( question )
 
-    ## Cas 1 : un seul énoncé, QCM
-    cat( "  QCM,", n.reponses, "r\u00e9ponses possibles\n" )
-
     if ( is.na( colonne.note ) ) {
         erreur( 1011, "convertir_question",
                 "Pr\u00e9cisez une colonne avec les notes",
@@ -998,6 +1088,9 @@ convertir_question <- function( question,
         question[ which( question[ , colonne.note ] %in% c( "v", "true", "t", "vrai" ) ), colonne.note ] <- 100 / nb.vrai
     }
     question[ , colonne.note ] <- as.numeric( question[ , colonne.note ] )
+    lst <- get( "df.convertie", envir = SARP.Moodle.env)
+    lst$QCM$df <- question
+    assign( "df.convertie", lst, envir = SARP.Moodle.env )
 
     ## On vérifie qu'elles sont entre 0 et 100
     ##   Remarque : plus basse note possible dans moodle : 5 % (soit 0,05),
@@ -1005,7 +1098,7 @@ convertir_question <- function( question,
     idx <- which( ( abs( question[ , colonne.note ] ) <= 1 ) & ( question[ , colonne.note ] != 0 ) )
     if ( length( idx ) > 0 ) {
         avertissement( 1005, "convertir_question",
-                       "Attention, des notes comprises entre 0 et 1",
+                       "Des notes comprises entre 0 et 1",
                        " ont \u00e9t\u00e9 converties en pourcentages." )
         question[ idx , colonne.note ] <- 100.0 * question[ idx , colonne.note ]
     }
@@ -1013,7 +1106,60 @@ convertir_question <- function( question,
     ## On distingue les bonnes et les mauvaises réponses
     bonnes <- which( question[ , colonne.note ] > 0 )
     mauvaises <- setdiff( 1:n.reponses, bonnes )
+    if ( length( bonnes ) == 0 ) {
+        lst$QCM$Bonnes    <- bonnes
+        lst$QCM$Mauvaises <- mauvaises
+        assign( "df.convertie", lst, envir = SARP.Moodle.env )
+        erreur( 350, "convertir_question",
+                "Aucune bonne r\u00e9ponse indiqu\u00e9e..." )
+    }
 
+    ## Réponses numériques multiples
+    if ( type.question %in% c( "n", "num\u00e9rique", "numerique", "nombre" ) ) {
+        cat( "  Num\u00e9rique,", n.reponses, "r\u00e9ponses possibles" )
+
+        if ( length( mauvaises ) > 0 ) {
+            avertissement( 1009, "convertir_question",
+                           "Les mauvaises r\u00e9ponses num\u00e9riques",
+                           " seront ignor\u00e9es" )
+        }
+        
+        ## On construit les réponses
+        reponses <- trouver_reponse.numerique( question[ bonnes, colonne.reponse ],
+                                               "convertir_question" )
+        if ( any( is.na( reponses$Nombre ) ) ) {
+            erreur( 1018, "convertir_question",
+                    "Question num\u00e9rique demand\u00e9e,",
+                    " mais r\u00e9ponse non-num\u00e9rique [",
+                    paste0( "\"", 
+                            reponses$Nombre[ which( is.na( reponses$Nombre ) ) ],
+                            "\"", collapse = " ; " ),
+                    , "]" )
+        }
+
+        cat( sep = "", " {",
+             paste0( reponses$Nombre,
+                     ifelse( is.finite( reponses$Tolerance ),
+                             paste0( "\u00b1 ", reponses$Tolerance ),
+                             "" ),
+                     collapse = " ; " ), "}",
+             "\n" )
+        
+        ## Et on crée la question...
+        numerique.moodle( texte         = question[ 1     , colonne.texte ],
+                          bonne.reponse = reponses$Nombre,
+                          notes         = question[ bonnes, colonne.note ],
+                          n.decimales   = reponses$Precision, 
+                          tolerance     = reponses$Tolerance, tolerance.type = 2,
+                          titre         = question[ 1     , colonne.titre ],
+                          fichier.xml = fichier.xml,
+                          commentaire.global = commentaire.global,
+                          note.question = note.question, penalite = penalite.question,
+                          temps = temps )
+
+        return( "Num\u00e9rique" )
+    }
+    
     ## Y a-t-il une seule bonne réponse, ou plusieurs ?
     ## reponse.unique <- FALSE
     ## if ( FALSE == is.na( type.question ) ) {
@@ -1042,21 +1188,29 @@ convertir_question <- function( question,
         question[ mauvaises, colonne.note ] <- - 100 / length( mauvaises )
     }
 
+    ## Cas 1 : un seul énoncé, QCM
+    cat( "  QCM,", n.reponses, "r\u00e9ponses possibles",
+         if ( all( !is.na( type.question ),
+                   type.question == 'qci' ) ) " [notation internat]",
+         "\n" )
+
     ## Et on crée la question...
     qcm.moodle( texte = question[ 1, colonne.texte ],
-               bonnes.reponses    = question[ bonnes   , colonne.reponse ],
-               mauvaises.reponses = question[ mauvaises, colonne.reponse ],
-               fractions = list( "Bonnes"  = question[ bonnes, colonne.note ],
-                                "Fausses" = question[ mauvaises, colonne.note ] ),
-               titre = question[ 1, colonne.titre ],
-               unique = if ( forcer.multiple ) FALSE else reponse.unique,
-               melanger = melanger.reponses,
-               fichier.xml = fichier.xml,
-               commentaire.global = commentaire.global,
-               note.question = note.question, penalite = penalite.question,
-               temps = temps )
+                bonnes.reponses    = question[ bonnes   , colonne.reponse ],
+                mauvaises.reponses = question[ mauvaises, colonne.reponse ],
+                fractions = list( "Bonnes"  = question[ bonnes, colonne.note ],
+                                  "Fausses" = question[ mauvaises, colonne.note ] ),
+                titre = question[ 1, colonne.titre ],
+                unique = if ( forcer.multiple ) FALSE else reponse.unique,
+                melanger = melanger.reponses,
+                fichier.xml = fichier.xml,
+                commentaire.global = commentaire.global,
+                note.question = note.question, penalite = penalite.question,
+                temps = temps,
+                internat = if ( is.na( type.question ) ) FALSE else ( type.question == 'qci' ) )
 
-    return( "QCM ou QCU" )
+    return( if ( all( !is.na( type.question ),
+                      type.question == 'qci' ) ) "QCM Internat" else "QCM ou QCU" )
 }
 
 
