@@ -17,7 +17,10 @@
 ##   20 mai  2020 : pour latex, possibilité de le faire via le filtre Moodle
 ##                    plutôt que par une image
 ##
-##   18 mai   2023 : conversion stop → erreur
+##   18 mai  2023 : conversion stop → erreur
+##
+##   10 jui. 2023 : ajouté l'option "halt on error" de latex
+##                  mode de fonctionnement = batch (aucun affichage, pas de question)
 ## —————————————————————————————————————————————————————————————————
 
 ## —————————————————————————————————————————————————————————————————
@@ -29,7 +32,7 @@
 inserer_formule.moodle <- function( formule, filtre.Moodle = FALSE, displaystyle = TRUE,
                                     marge = 2,
                                     couleurs = TRUE, enjoliver = TRUE,
-                                    cmd.latex = "latex -shell-escape --interaction errorstopmode",
+                                    cmd.latex = getOption( "Sm.cmd_latex" ),
                                     packages = c( "amsfonts", "amsmath",
                                                   "amssymb" , "dsfont" ),
                                     options.convert = list( 'density' = 150,
