@@ -28,7 +28,12 @@
 ##
 ##    3 jui. 2022 : adaptation pour utiliser le temps de catégorie
 ##
-##   18 mai   2023 : conversion stop → erreur
+##   18 mai  2023 : conversion stop → erreur
+##
+##   24 juin 2023 : essais de distracteurs sur le glisser-déposer de texte
+##                  préparé pour les tags de questions
+##
+##   15 avr. 2024 : réparé une fausse manip ayant fusionné legender_image et glisser_texte
 ## —————————————————————————————————————————————————————————————————
 
 ## —————————————————————————————————————————————————————————————————
@@ -46,7 +51,7 @@ glisser_deposer.moodle <- function( texte, titre = "Glisser-d\u00e9poser...",
                                     grp.zones = rep( 1, n.zones ),
                                     ordre.aleatoire = TRUE,
                                     commentaire.global = NA, penalite = NA, note.question = NA,
-                                    idnum = NA, temps,
+                                    idnum = NA, temps, tags = NULL,
                                     fichier.xml = get( "fichier.xml", envir = SARP.Moodle.env ),
                                     ... )
 {
@@ -57,7 +62,7 @@ glisser_deposer.moodle <- function( texte, titre = "Glisser-d\u00e9poser...",
     ## On commence la question
     debut_question.moodle( type = "ddimageortext",
                            titre = titre, texte = texte,
-                           penalite = penalite, note = note.question,
+                           penalite = penalite, note.question = note.question,
                            commentaire.global = commentaire.global,
                            idnum = idnum,
                            fichier.xml = fichier.xml )
@@ -186,7 +191,7 @@ glisser_deposer.moodle <- function( texte, titre = "Glisser-d\u00e9poser...",
     }
     
     ## On a fini la question
-    fin_question.moodle( fichier.xml = fichier.xml  )
+    fin_question.moodle( fichier.xml = fichier.xml, tags = tags )
 }
 
 
@@ -203,7 +208,7 @@ legender_image.moodle <- function( texte, titre = "L\u00e9gender...",
                                    ordre.aleatoire = TRUE, afficher.erreurs = TRUE,
                                    commentaire.global = NA, penalite = NA, note.question = NA,
                                    idnum = NA,
-                                   temps,
+                                   temps, tags = NULL,
                                    fichier.xml = get( "fichier.xml", envir = SARP.Moodle.env ),
                                    ... )
 {
@@ -214,7 +219,7 @@ legender_image.moodle <- function( texte, titre = "L\u00e9gender...",
     ## On commence la question
     debut_question.moodle( type = "ddmarker",
                            titre = titre, texte = texte,
-                           penalite = penalite, note = note.question,
+                           penalite = penalite, note.question = note.question,
                            commentaire.global = commentaire.global,
                            idnum = idnum,
                            fichier.xml = fichier.xml )
@@ -407,7 +412,7 @@ legender_image.moodle <- function( texte, titre = "L\u00e9gender...",
     }
     
     ## On a fini la question
-    fin_question.moodle( fichier.xml = fichier.xml  )
+    fin_question.moodle( fichier.xml = fichier.xml, tags = tags )
 }
 
 
@@ -419,9 +424,10 @@ legender_image.moodle <- function( texte, titre = "L\u00e9gender...",
 
 glisser_textes.moodle <- function( texte, titre = "Glisser les textes...",
                                    groupe = rep( 1, n.zones ), infini = FALSE,
+                                   distracteurs = NULL,
                                    ordre.aleatoire = TRUE, afficher.erreurs = TRUE,
                                    commentaire.global = NA, penalite = NA, note.question = NA,
-                                   idnum = NA, temps,
+                                   idnum = NA, temps, tags = NULL,
                                    fichier.xml = get( "fichier.xml", envir = SARP.Moodle.env ) )
 {
     ## On prépare l'énoncé et les zones
@@ -455,7 +461,7 @@ glisser_textes.moodle <- function( texte, titre = "Glisser les textes...",
     ## On commence la question
     debut_question.moodle( type = "ddwtos",
                            titre = titre, texte = texte,
-                           penalite = penalite, note = note.question,
+                           penalite = penalite, note.question = note.question,
                            commentaire.global = commentaire.global,
                            idnum = idnum,
                            fichier.xml = fichier.xml )
@@ -492,7 +498,26 @@ glisser_textes.moodle <- function( texte, titre = "Glisser les textes...",
              if ( infini[ idx ] ) "      <infinite/>\n",
              "    </dragbox>\n" )
     }
+
+    ## Gestion des distracteurs
+    if ( ! is.null( distracteurs ) ) {
+        if ( is.vector( distracteurs ) ) {
+            distracteurs <- data.frame( "Textes" = distracteurs,
+                                        "Groupe" = 1,
+                                        "Infini" = FALSE )
+        }
+
+        for ( i in 1:nrow( distracteurs ) ) {
+            cat( file = fichier.xml, sep = "",
+                 "    <dragbox>\n" )
+            coder.texte( distracteurs$Textes[ i ], fichier.xml = fichier.xml )
+            cat( file = fichier.xml, sep = "",
+                 "      <group>", distracteurs$Groupe[ i ], "</group>\n",
+                 if ( distracteurs$Infini[ i ] ) "      <infinite/>\n",
+                 "    </dragbox>\n" )            
+        }
+    }
     
     ## On a fini la question
-    fin_question.moodle( fichier.xml = fichier.xml  )
+    fin_question.moodle( fichier.xml = fichier.xml, tags = tags )
 }
