@@ -22,7 +22,9 @@
 ##                     affichage des messages par cat et plus print
 ##                     affichage des messages de choix de dossier désactivable
 ##
-##   18   mai      2023 : conversion stop → erreur et warning → avertissement
+##   18 mai     2023 : conversion stop → erreur et warning → avertissement
+##
+##   12 janvier 2024 : corrigé le cas d'un dossier image Moodle
 ## —————————————————————————————————————————————————————————————————
 
 definir_dossier.image.moodle <- function( URL, local = FALSE,
@@ -69,7 +71,7 @@ definir_dossier.image.moodle <- function( URL, local = FALSE,
         }
         assign( "URL_base", URL, envir = SARP.Moodle.env )
 
-        r <- dossiers
+        r <- URL
     }
 
     ## On renvoie l'URL ou la liste des dossiers de recherche
