@@ -42,6 +42,8 @@
 ##    3 juillet 2022 : adaptation pour utiliser le temps de catégorie
 ##
 ##   18 mai     2023 : conversion stop → erreur et warning → avertissement
+##
+##   23 avril   2024 : gestion des tags
 ## ─────────────────────────────────────────────────────────────────
 
 ## ─────────────────────────────────────────────────────────────────
@@ -462,7 +464,7 @@ question_libre.moodle <- function( texte.intro, textes.avant, texte.final,
                                    reponses, notes = NULL, types = NULL, commentaires = NULL,
                                    titre = "Question libre",
                                    commentaire.global = NA, penalite = NA, note.question = NA,
-                                   idnum = NA, temps,
+                                   idnum = NA, temps, tags = NULL,
                                    fichier.xml = get( "fichier.xml", envir = SARP.Moodle.env ) ) {
     ## Combien de questions au total ?
     if( length( textes.avant ) != length( reponses ) ) {
@@ -525,6 +527,7 @@ question_libre.moodle <- function( texte.intro, textes.avant, texte.final,
     ## On crée la question
     question.moodle( fichier.xml = fichier.xml, type = "cloze",
                      titre = titre, texte = texte, reponses = NULL,
-                     penalite = penalite, note = note.question,
-                     commentaire.global = commentaire.global, idnum = idnum )
+                     penalite = penalite, note.question = note.question,
+                     commentaire.global = commentaire.global, idnum = idnum,
+                     tags = tags )
 }
