@@ -10,6 +10,9 @@
 ##    1 jan. 2021 : question de pur texte (« description »)
 ##
 ##    3 jui. 2022 : corrigé le titre pour les questions description
+##
+##   15 avr. 2024 : option pour transformer les insécables en espace
+##                   dans preparer_textes [pour les notes…]
 ## —————————————————————————————————————————————————————————————————
 
 ## —————————————————————————————————————————————————————————————————
@@ -20,18 +23,18 @@
 description.moodle <- function( texte,
                                 titre = "Description",
                                 commentaire.global = NA, 
-                                idnum = NA, 
+                                idnum = NA, tags = NULL,
                                 fichier.xml = get( "fichier.xml", envir = SARP.Moodle.env ) )
 {
     ## On crée la question
     debut_question.moodle( type = "description",
                            titre = titre, texte = texte,
                            commentaire.global = commentaire.global,
-                           penalite = 0, note = 0, idnum = idnum,
+                           penalite = 0, note.question = 0, idnum = idnum,
                            fichier.xml = fichier.xml )
 
     ## Et on a fini en fait…
-    fin_question.moodle( fichier.xml = fichier.xml )
+    fin_question.moodle( fichier.xml = fichier.xml, tags = tags )
 }
 
 ## —————————————————————————————————————————————————————————————————
@@ -40,8 +43,12 @@ description.moodle <- function( texte,
 ## 
 ## —————————————————————————————————————————————————————————————————
 
-preparer_texte <- function( textes )
+preparer_texte <- function( textes, oter.insecables = FALSE )
 {
+    if ( oter.insecables ) {
+        textes <- gsub( "\u00a0", "", fixed = TRUE, textes )
+    }
+    
     ## On supprime les doubles-espaces
     textes <- gsub( " {2,}", " ", textes )
 
